"use strict";
const fary_log_1 = require("fary-log");
const Koa = require("koa");
const koaCompress = require("koa-compress");
const koaLogger = require("koa-logger");
const error_1 = require("./middlewares/error");
const proxy_1 = require("./middlewares/proxy");
const check_proxy_1 = require("./utils/check-proxy");
const env_1 = require("./utils/env");
const server_config_1 = require("./utils/server-config");
const logger = fary_log_1.default('fary:app');
const DEFAULT_OPTIONS = {
    middlewares: [],
};
/**
 * createApp
 * @param {Object} options - options
 * @param {Function} onStart - callback after server start
 * @return {Koa} app - serverApp
 */
function createApp(options, onStart = () => void 0) {
    options = Object.assign({}, DEFAULT_OPTIONS, options);
    if (env_1.IS_DEV) {
        check_proxy_1.default(server_config_1.default.proxy);
    }
    logger.log(`Debug mode:`, env_1.IS_DEV);
    logger.log(`Proxy mapping:`, server_config_1.default.proxy);
    const app = new Koa();
    // trust proxy
    app.proxy = true;
    const middlewares = [
        // request/response log
        koaLogger(),
        // compress the response
        koaCompress(),
        // error handler
        error_1.default(),
        ...options.middlewares,
        // proxy handler
        proxy_1.default(),
    ];
    middlewares.forEach((middleware) => {
        if (!middleware) {
            return;
        }
        app.use(middleware);
    });
    app.on('error', (err) => {
        logger.error(`Server error: \n ${err.stack}`);
    });
    app.listen(server_config_1.default.port, () => {
        logger.info(`Koa server is listening at ${server_config_1.default.port}`);
        onStart();
    });
    process.on('SIGINT', () => {
        process.exit();
    });
    return app;
}
module.exports = createApp;
